/**
 * Audio player class for native app
 * In this class, we use the phonegap api 
 * and the native api for implementation
 * of the audio-API (play, getVolume, setUrl, etc.)
 * 
 * @author Matanya
 */
var audioDriver = (function ()
{

    var me = {};

    // The volume. number 0.0 (min) - 1.0 (max)
    var _volume = 1.0;

    // "running number", that we increase every time we create a new "player" (=audio tag).
    var _idCounter = -1;

    // dictionary of all media players (the player-handle is index to this array)
    var _players = {};

    // A map of event-handlers (that registered to "volumeChanged")
    var _volumeChangedHandler = {};

    // A map of event-handlers (that registered to "mediaEnded")
    var _mediaEndedHandler = {};

    // A map of event-handlers (that registered to "mediaPlaying")
    var _mediaPlayingHandler = {};

    // A map of event-handlers (that registered to "mediaPause")
    var _mediaPauseHandler = {};

    // A map of event-handlers (that registered to "mediaTimeUpdate")
    var _mediaTimeUpdateHandler = {};

    // A map of event-handlers (that registered to "mediaCanPlay")
    var _mediaCanPlayHandler = {};

    // A map of event-handlers (that registered to "mediaSeeked")
    var _mediaSeekedHandler = {};

    // A map of event-handlers (that registered to "mediaSeeking")
    var _mediaSeekingHandler = {};

    // A map of event-handlers (that registered to "loadedMetaData")
    var _loadedMetaDataHandler = {};

    // A map of event-handlers (that registered to "loadStart")
    var _loadStartHandler = {};

    // A map of event-handlers (that registered to "error")
    var _errorHandler = {};

    // Boolean: is audio-tag supported in this browser?
    var _isAudioSupported = !!document.createElement('audio').canPlayType;

    me.isSupported = function ()
    {
        return true;
    };

    /** 
    * Get the volume of the music.
    *
    * @return number 0.0 (min) to 1.0 (max) that represents the volume. (NOTE: not really return, it will be parameter for "cbSuccess"!)
    * @author Matanya
    **/
    me.getVolume = function (cbSuccess, cbFail)
    {
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess(_volume);
        }
    };

    /** 
    * Change the volume of the music.
    *
    * @param volume number 0.0 (min) to 1.0 (max) that represents the volume
    * @author Matanya
    **/
    me.setVolume = function (cbSuccess, cbFail, volume)
    {
        _volume = volume;

        // Change all audio tags:
        var $allAudioTags = $("audio");
        $.each($allAudioTags, function (i, audioTag)
        {
            audioTag.volume = volume;
        });

		// Call all registered handlers (in case we registered from the appbase and not from the pages).
        $.each(_volumeChangedHandler, function (i, handler)
        {
            handler(volume);
        });
		
		// notify the pages of the voulume change.
		eventsManager.dispatchEvent('volumeChangedEvent', {volume: volume});

        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Register to volume-change event
    *
    * @param context - context of the calling api
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerVolumeChanged = function (cbSuccess, cbFail, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _volumeChangedHandler[context.pageId] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to volume-change event for a specific context
    *
    * @param context - context of the calling api
    * 
    * @author Matanya
    **/
    me.unregisterVolumeChanged = function (cbSuccess, cbFail, context)
    {
        if (context.pageId)
        {
            if (_volumeChangedHandler[context.pageId])
            {
                delete _volumeChangedHandler[context.pageId];
                if (typeof (cbSuccess) === 'function')
                {
                    cbSuccess();
                    return;
                }
            }
			// in case this is 
			else 
			{
				cbSuccess();
			}
        }
        if (typeof (cbFail) === 'function')
        {
            cbFail();
        }
    };

    /**
    * Register to onEnded event of the audio
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerMediaEnded = function (cbSuccess, cbFail, playerHandle, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _mediaEndedHandler[playerHandle] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to onEnded event of the audio for a specific context
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * 
    * @author Matanya
    **/
    me.unregisterMediaEnded = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (context.pageId)
        {
            if (_mediaEndedHandler[playerHandle])
            {
                delete _mediaEndedHandler[playerHandle];
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Register to onPlaying event of the audio
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerMediaPlaying = function (cbSuccess, cbFail, playerHandle, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _mediaPlayingHandler[playerHandle] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to onPlaying event of the audio for a specific context
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * 
    * @author Matanya
    **/
    me.unregisterMediaPlaying = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (context.pageId)
        {
            if (_mediaPlayingHandler[playerHandle])
            {
                delete _mediaPlayingHandler[playerHandle];
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Register to onPause event of the audio
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerMediaPause = function (cbSuccess, cbFail, playerHandle, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _mediaPauseHandler[playerHandle] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to onPause event of the audio for a specific context
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * 
    * @author Matanya
    **/
    me.unregisterMediaPause = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (context.pageId)
        {
            if (_mediaPauseHandler[playerHandle])
            {
                delete _mediaPauseHandler[playerHandle];
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Register to onTimeUpdate event of the audio
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerMediaTimeUpdate = function (cbSuccess, cbFail, playerHandle, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _mediaTimeUpdateHandler[playerHandle] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to onTimeUpdate event of the audio for a specific context
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * 
    * @author Matanya
    **/
    me.unregisterMediaTimeUpdate = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (context.pageId)
        {
            if (_mediaTimeUpdateHandler[playerHandle])
            {
                delete _mediaTimeUpdateHandler[playerHandle];
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Register to onCanPlay event of the audio
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerMediaCanPlay = function (cbSuccess, cbFail, playerHandle, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _mediaCanPlayHandler[playerHandle] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to onCanPlay event of the audio for a specific context
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * 
    * @author Matanya
    **/
    me.unregisterMediaCanPlay = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (context.pageId)
        {
            if (_mediaCanPlayHandler[playerHandle])
            {
                delete _mediaCanPlayHandler[playerHandle];
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Register to onSeeked event of the audio
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerMediaSeeked = function (cbSuccess, cbFail, playerHandle, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _mediaSeekedHandler[playerHandle] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to onSeeked event of the audio for a specific context
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * 
    * @author Matanya
    **/
    me.unregisterMediaSeeked = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (context.pageId)
        {
            if (_mediaSeekedHandler[playerHandle])
            {
                delete _mediaSeekedHandler[playerHandle];
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Register to onSeeking event of the audio
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerMediaSeeking = function (cbSuccess, cbFail, playerHandle, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _mediaSeekingHandler[playerHandle] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to onSeeking event of the audio for a specific context
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * 
    * @author Matanya
    **/
    me.unregisterMediaSeeking = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (context.pageId)
        {
            if (_mediaSeekingHandler[playerHandle])
            {
                delete _mediaSeekingHandler[playerHandle];
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Register to loadedMetaData event of the audio
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerLoadedMetaData = function (cbSuccess, cbFail, playerHandle, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _loadedMetaDataHandler[playerHandle] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to loadedMetaData event of the audio for a specific context
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * 
    * @author Matanya
    **/
    me.unregisterLoadedMetaData = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (context.pageId)
        {
            if (_loadedMetaDataHandler[playerHandle])
            {
                delete _loadedMetaDataHandler[playerHandle];
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Register to loadStart event of the audio
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerLoadStart = function (cbSuccess, cbFail, playerHandle, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _loadStartHandler[playerHandle] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to loadStart event of the audio for a specific context
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * 
    * @author Matanya
    **/
    me.unregisterLoadStart = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (context.pageId)
        {
            if (_loadStartHandler[playerHandle])
            {
                delete _loadStartHandler[playerHandle];
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Register to error event of the audio
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * @param cbEventHandler - callback to handle the event
    * 
    * @author Matanya
    **/
    me.registerError = function (cbSuccess, cbFail, playerHandle, cbEventHandler, context)
    {
        if (typeof (cbEventHandler) === 'function')
        {
            if (context.pageId)
            {
                _errorHandler[playerHandle] = cbEventHandler;
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /**
    * Remove registration to error event of the audio for a specific context
    *
    * @param context - context of the calling api
    * @param playerHandle - integer, the handle of the player.
    * 
    * @author Matanya
    **/
    me.unregisterError = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (context.pageId)
        {
            if (_errorHandler[playerHandle])
            {
                delete _errorHandler[playerHandle];
            }
        }
        if (typeof (cbSuccess) === 'function')
        {
            cbSuccess();
        }
    };

    /** 
    * Create a new audio-player, and return the "handle" of it.
    *
    * @param context - context of the calling api
    *
    * @return handle native number that is handle to this new player. (NOTE: not really return, it will be parameter for "cbSuccess"!)
    * @author Matanya
    **/
    me.createPlayer = function (cbSuccess, cbFail, context)
    {
		// Find new ID:
        _idCounter++;
		
        // The new "handler:
        var playerHandle = _idCounter;
	
        // playerHandle (= _idCounter) will be the "handle" to this player:
        Utils.Helpers.funcWrapper(cbSuccess)(playerHandle);
    };

    /** 
    * Change the currently played audio URL
    *
    * @param playerHandle - integer, the handle of the player.
    * @param url - string, the new URL to play.
    * @param context - context of the calling api
    * @author Matanya
    **/
    me.setUrl = function (cbSuccess, cbFail, playerHandle, url, context)
	{
		var cbMediaSuccess = function()
		{
 			Utils.Helpers.funcWrapper(_loadedMetaDataHandler[playerHandle])();
			eventsManager.dispatchEvent('loadedMetaDataEvent', 
					{playerHandle: playerHandle});			
		}; 
		// Only here we will create the media - because we need the src...
		var my_media = new Media(url, cbMediaSuccess, cbFail);
		
		if (_players[playerHandle])
		{
			_players[playerHandle].release();
		}
		
		_players[playerHandle] = my_media;
		
        Utils.Helpers.funcWrapper(cbSuccess)();		
    };

    /** 
    * Change the current-time position (seek)
    *
    * @param playerHandle - integer, the handle of the player.
    * @param currentTime - double, the new playback position, in seconds.
    * @param context - context of the calling api
    * @author Matanya
    **/
    me.setCurrentTime = function (cbSuccess, cbFail, playerHandle, currentTime, context)
	{
		//TBD...
    };

    /** 
    * Get the current-time position (seek)
    *
    * @param playerHandle - integer, the handle of the player.
    * @param context - context of the calling api
    * @return currentTime - double, the playback 
    * 	 position, in seconds. (NOTE: not really return, 
    *	 it will be parameter for "cbSuccess"!)
    * @author Matanya
    **/
    me.getCurrentTime = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (_players[playerHandle])
		{
			_players[playerHandle].getCurrentPosition(
            	// success callback
            	function(position) 
				{
                	Utils.Helpers.funcWrapper(cbSuccess)(position/1000);
                }
            	,
            	// error callback
            	function(e) 
				{
             		Utils.Helpers.funcWrapper(cbFail)();
            	});
        }
        else 
        {
             Utils.Helpers.funcWrapper(cbFail)();
        }
    };

    /** 
    * Get the duration-time 
    *
    * @param playerHandle - integer, the handle of the player.
    * @param context - context of the calling api
    * @return durationTime - double, the duration 
    * 	 time, in seconds. (NOTE: not really return, 
    *	 it will be parameter for "cbSuccess"!)
    * @author Matanya
    **/
    me.getDuration = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (_players[playerHandle])
        {
             Utils.Helpers.funcWrapper(cbSuccess)(_players[playerHandle].getDuration()/1000);
        }
        else 
        {
             Utils.Helpers.funcWrapper(cbFail)();
        }
    };

    /** 
    * Pause the currently playing audio.
    *
    * @param playerHandle - integer, the handle of the player.
    * @param context - context of the calling api
    * @author Matanya
    **/
    me.pause = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (_players[playerHandle])
		{
			_players[playerHandle].pause();
            Utils.Helpers.funcWrapper(cbSuccess)();
			
			// TEMP!! Simulate mediaPlayingEvent:
            Utils.Helpers.funcWrapper(_mediaPlayingHandler[playerHandle])();
			eventsManager.dispatchEvent('mediaPlayingEvent', {playerHandle: playerHandle});				
        }
        else 
        {
        	Utils.Helpers.funcWrapper(cbFail)();
        }
    };

    /** 
    * Play the source (= url) that was already loaded to the player.
    *
    * @param playerHandle - integer, the handle of the player.
    * @param context - context of the calling api
    * @author Matanya
    **/
    me.play = function (cbSuccess, cbFail, playerHandle, context)
    {
        if (_players[playerHandle])
		{
			_players[playerHandle].play();
            Utils.Helpers.funcWrapper(cbSuccess)();

			// TEMP!! Simulate mediaPlayingEvent:
			setTimeout(function() 
			{
	            Utils.Helpers.funcWrapper(_mediaPlayingHandler[playerHandle])();
				eventsManager.dispatchEvent('mediaPlayingEvent', {playerHandle: playerHandle});				
    		}, 2000);        

			// TEMP!! Simulate mediaTimeUpdateEvent:
			setInterval(function() 
			{
				if (_players[playerHandle])
				{
					_players[playerHandle].getCurrentPosition(
            		// success callback
            		function(position) 
					{
			            Utils.Helpers.funcWrapper(_mediaTimeUpdateHandler[playerHandle])();
						eventsManager.dispatchEvent('mediaTimeUpdateEvent', 
							{playerHandle: playerHandle,
							currentTime: position/1000});				
                	}
            		,
            		// error callback
            		null);
        		}
    		}, 1000);        
			
        }
        else 
        {
        	Utils.Helpers.funcWrapper(cbFail)();
        }
    };

    return me;
} ());
